const Frequency = require('./Frequency');
const MenuAnalytics = require('./MenuAnalytics');
const { getWeek } = require('../Utils/DateUtil');
const moment = require('moment');
const ServiceConfig = require('./ServiceConfig');

class Rules {
  setAnalytic(data) {
    Rules.analytic = data;
  }

  getAnalytic(menuDate) {
    return Rules.analytic[menuDate];
  }

  ingredientDay(dish, otherInfo) {
    const { INGR_PRINCIPAL_1, INGR_PRINCIPAL_2 } = dish;
    const { ingredient = {} } = otherInfo.today || {};
    if (MenuAnalytics.hasGlobalCharacteristic(ingredient, INGR_PRINCIPAL_1)) return false;
    if (MenuAnalytics.hasGlobalCharacteristic(ingredient, INGR_PRINCIPAL_2)) return false;
    return true;
  }

  ingredientConsecutiveDays(dish, otherInfo) {
    const { INGR_PRINCIPAL_1, INGR_PRINCIPAL_2, COD_TIP_PRATO } = dish;
    const { yesterday, tomorrow } = otherInfo;
    if (yesterday) {
      if (MenuAnalytics.hasLocalCharacteristic(yesterday.ingredient, INGR_PRINCIPAL_1, COD_TIP_PRATO)) return false;
      if (MenuAnalytics.hasLocalCharacteristic(yesterday.ingredient, INGR_PRINCIPAL_2, COD_TIP_PRATO)) return false;
    }
    if (tomorrow) {
      if (MenuAnalytics.hasLocalCharacteristic(tomorrow.ingredient, INGR_PRINCIPAL_1, COD_TIP_PRATO)) return false;
      if (MenuAnalytics.hasLocalCharacteristic(tomorrow.ingredient, INGR_PRINCIPAL_2, COD_TIP_PRATO)) return false;
    }
    return true;
  }

  colorDay(dish, otherInfo) {
    const { today, sequential } = otherInfo;
    const { ignoreColor } = ServiceConfig.getConfig(sequential);
    if(ignoreColor) return true;
    const { COD_TIP_PRATO, COR_PRATO } = dish;
    if (today && COR_PRATO) {
      const { color = {} } = today;
      if (MenuAnalytics.hasLocalCharacteristic(color, COR_PRATO, COD_TIP_PRATO)) return false;
    }
    return true;
  }

  prepModeDay(dish, otherInfo) {
    const { today, sequential } = otherInfo;
    const { ignorePrepMode } = ServiceConfig.getConfig(sequential);
    if(ignorePrepMode) return true;
    const { MODO_PREPARO, COD_TIP_PRATO } = dish;
    if (today && MODO_PREPARO) {
      const { prepMode = {} } = today;
      return MenuAnalytics.hasLocalCharacteristic(prepMode, MODO_PREPARO, COD_TIP_PRATO);
    }
    return true;
  }

  hasAvailableFreqMonth(freq) {
    const { month = 0, maxMonth } = freq;
    if(maxMonth && month >= maxMonth) return false;
    return true;
  }

  hasAvailableFreqWeek(freq, weekNumber) {
    const { week = {}, maxWeek } = freq;
    const freqWeek = week[weekNumber] || 0;
    if(maxWeek && freqWeek >= maxWeek) return false;
    return true;
  }

  freqGroupMonth(dish, otherInfo) {
    const freq = Frequency.groupFreq[dish.gpprato] || {};
    return this.hasAvailableFreqMonth(freq);
  }

  freqGroupWeek(dish, otherInfo) {
    const freq = Frequency.groupFreq[dish.gpprato] || {};
    const weekNumber = getWeek(otherInfo.menuDate);
    return this.hasAvailableFreqWeek(freq, weekNumber);
  }

  freqDishMonth(dish, otherInfo) {
    const freq = Frequency.dishFreq[dish.prato] || {};
    return this.hasAvailableFreqMonth(freq);
  }

  freqDishWeek(dish, otherInfo) {
    const freq = Frequency.dishFreq[dish.prato] || {};
    const weekNumber = getWeek(otherInfo.menuDate);
    return this.hasAvailableFreqWeek(freq, weekNumber);
  }

  restrictionUtilizationGroup(dish, otherInfo) {
    const { menuDate, utilizationGroupWeek, calendar } = otherInfo;
    const dayOfWeek = {
      1: 'IND_SEGUNDA',
      2: 'IND_TERCA',
      3: 'IND_QUARTA',
      4: 'IND_QUINTA',
      5: 'IND_SEXTA',
      6: 'IND_SABADO',
      7: 'IND_DOMINGO',
    };
    const ut = utilizationGroupWeek.find((ut) => ut.NUM_GRP_PRATO == dish.gpprato);
    if(!ut) return true;
    const menuDateInstance = moment(menuDate, 'DD/MM/YYYY');
    const weekDay = menuDateInstance.isoWeekday();
    const day = calendar[Number(menuDateInstance.format('DD'))-1];
    const isRestrictByWeekDay = ut[dayOfWeek[weekDay]] === 'R';
    if(isRestrictByWeekDay) return false;
    const dayInfo = [day.IDPRIMSEM, day.IDSEMANA2, day.IDSEMANA3, day.IDSEMANA4, day.IDULTMSEM];
    const isRestrictByWeekInMonth = [ut.IDPRIMSEM, ut.IDSEMANA2, ut.IDSEMANA3, ut.IDSEMANA4, ut.IDULTMSEM].some((restrict, idx) => {
      return restrict === 'R' && dayInfo[idx] === 'S';
    });
    if (isRestrictByWeekInMonth || (day.IDFERIADO === 'S' && ut.IDFERIADO === 'R')) return false;

    return true;
  }

  restrictionDishAndGroupComb(dish, otherInfo) {
    const { restComb, todayMenu } = otherInfo;
    const isValidRestrictions = restComb.every(res => {
      const isRestrictionToDish = res.COD_GRP_TIP_PRATO_COMB === dish.COD_GRP_TIP_PRATO &&
                                 (res.COD_PRATO_COMB === dish.prato || res.NRGRUPOPRATOSERV_COMB === dish.gpprato);
      if(isRestrictionToDish) {
        const hasRestrictor = todayMenu.some( dishOnDay => {
          return dishOnDay.COD_GRP_TIP_PRATO === res.COD_GRP_TIP_PRATO &&
                (dishOnDay.prato === res.COD_PRATO || dishOnDay.gpprato === res.NRGRUPOPRATOSERV)
        });
        return !hasRestrictor;
      }
      return true;
    });
    return isValidRestrictions;
  }
}

module.exports = new Rules();
